local ADDON_NAME, options = ...

local addon = _G.NamePlateKAI
if not addon then return end

local mainPanel = addon.optionsPanel
if not mainPanel then return end

local filterPanel = CreateFrame("Frame")
filterPanel.name = _G.FILTERS
filterPanel.parent = mainPanel.name

local function print(...)
	_G.print(YELLOW_FONT_COLOR_CODE..mainPanel.title..":"..FONT_COLOR_CODE_CLOSE, ...)
end

local GUI = LibStub and LibStub("AceGUI-3.0", true)
if not GUI then return print("AceGUI-3.0 was not found.") end
if not (LibStub and LibStub("AceDB-3.0", true)) then return print("AceDB-3.0 was not found.") end
local db = LibStub("AceDB-3.0"):New("NamePlateKAIDB", {profile = addon.defaults}, true)
local profile = db.profile

local FRAME_MARGIN = 14
local FRAME_SPACE = 4
local BARTEXTURE_DEFAULT = "NamePlateKAI"
local RELOAD = ORANGE_FONT_COLOR_CODE.."*"..REQUIRES_RELOAD..FONT_COLOR_CODE_CLOSE

local controls = {}
local filterControls = {}
local zoneFilters = {}
local userFilters = {}

local borderTypes = {
	{
		name = "Blur",
		file = "Interface\\AddOns\\NamePlateKAI\\texture\\glowTex",
		alpha = 0.8,
		GetCastBarOffset = function(edgeSize)
			return max(floor(edgeSize * 0.6), 3)
		end,
	},
	{
		name = "Sharp",
		file = "Interface\\AddOns\\NamePlateKAI\\texture\\border_sharp",
		alpha = 0.6,
		GetCastBarOffset = function(edgeSize)
			return max(floor(edgeSize * 0.9 + 0.5), 3)
		end,
	},
}

local function Animation_OnValueChanged(widget, event, value)
	local disabled = not controls.animationCheck:GetValue()
	controls.animationAlphaSlider:SetDisabled(disabled)
	controls.animationStyleDropdown:SetDisabled(disabled)
	controls.animationAlphaSlider:SetDisabled(disabled)
	controls.animationOverlayCheck:SetDisabled(disabled)
end

local function Shine_OnValueChanged(widget, event, value)
	local disabled = not controls.shineCheck:GetValue()
	controls.shineAlphaSlider:SetDisabled(disabled)
end

local function ExecuteColorCheck_OnValueChanged(widget, event, value)
	local disabled = not controls.healthTextCheck:GetValue() or controls.disableExecuteCheck:GetValue()
	controls.executeThresholdSlider:SetDisabled(disabled)
	controls.executeColor:SetDisabled(disabled)
end

local function HealthText_OnValueChanged(widget, event, value)
	local disabled = not controls.healthTextCheck:GetValue()
	controls.healthTextPercentageCheck:SetDisabled(disabled)
	controls.disableExecuteCheck:SetDisabled(disabled)
	controls.healthTextMaxCheck:SetDisabled(disabled)
	ExecuteColorCheck_OnValueChanged()
end

local function ColorPickerSetHexColor(colorPicker, hexColor)
	local r, g, b = hexColor:match("(%x%x)(%x%x)(%x%x)")
	if not (r and g and b) then return end
	r, g, b = tonumber("0x"..r) or 0xff, tonumber("0x"..g) or 0xff, tonumber("0x"..b) or 0xff
	colorPicker:SetColor(r / 0xff, g / 0xff, b / 0xff)
end

local function ColorPickerGetHexColor(colorPicker)
	return ("%.2x%.2x%.2x"):format(colorPicker.r * 0xff, colorPicker.g * 0xff, colorPicker.b * 0xff)
end

local function DropdownSetValue(dropdown, value, default)
	if value then
		for k, v in pairs(dropdown.list) do
			if v == value then
				dropdown:SetValue(k)
				break
			end
		end
	else
		dropdown:SetValue(default or _G.DEFAULT)
	end
end

function mainPanel:refresh()
	if not next(controls) then
		options:CreateWidget()
		options:CreateFiltersWidget()
	end

	controls.healthWidthSlider:SetValue(profile.frameSize.W)
	--controls.healthHeightSlider:SetValue(profile.frameSize.H)
	--controls.frameScaleHorizontal:SetValue(profile.frameScale.x)
	--controls.frameScaleVertical:SetValue(profile.frameScale.y)

	controls.healthHeightSlider:SetValue(profile.healthBarHeight)
	controls.castHeightSlider:SetValue(profile.castBarHeight)

	controls.frameMarginY:SetValue(profile.frameMargin.y)
	controls.raidIconSlider:SetValue(profile.raidIcon.size)
	controls.spellIconSlider:SetValue(profile.spellIcon.size)
	controls.borderWidthSlider:SetValue(profile.barBackdrop.edgeSize)

	local borderType = "blur"
	for k, v in pairs(borderTypes) do
		if v.file == profile.barBackdrop.edgeFile then
			borderType = k
		end
	end
	controls.borderStyleDropdown:SetValue(borderType)

	if controls.unitFontDropdown then
		DropdownSetValue(controls.unitFontDropdown, profile.nameFont.name)
		controls.unitNameSlider:SetValue(profile.nameFont.size)
		controls.unitNameOutlineCheck:SetValue(profile.nameFont.outline == "OUTLINE")
	end

	if controls.unitFontPlayerDropdown then
		DropdownSetValue(controls.unitFontPlayerDropdown, profile.nameFontPlayer.name)
		controls.unitNamePlayerSlider:SetValue(profile.nameFontPlayer.size)
		controls.unitNamePlayerOutlineCheck:SetValue(profile.nameFontPlayer.outline == "OUTLINE")
	end

	if controls.spellFontDropdown then
		DropdownSetValue(controls.spellFontDropdown, profile.castFont.name)
		controls.spellNameSlider:SetValue(profile.castFont.size)
		controls.spellNameOutlineCheck:SetValue(profile.castFont.outline == "OUTLINE")
	end

	if controls.healthTextureDropdown then
		DropdownSetValue(controls.healthTextureDropdown, profile.barTexture, BARTEXTURE_DEFAULT)
	end

	if controls.castTextureDropdown then
		DropdownSetValue(controls.castTextureDropdown, profile.castBarTexture, BARTEXTURE_DEFAULT)
	end

	controls.showDebuffCheck:SetValue(profile.buff.enabledEnemy)
	controls.showBuffCheck:SetValue(profile.buff.enabledFriendly)
	controls.showCountdownCheck:SetValue(profile.buff.countdownNumber)
	controls.showCountdownCheck:SetDisabled(not GetCVarBool("countdownForCooldowns"))
	controls.disableOmniccCheck:SetValue(profile.disableOmnicc)
	controls.disableOmniccCheck:SetDisabled(not _G.OmniCC)
	controls.auraHeightSlider:SetValue(profile.buff.size.H)
	controls.auraBorderSlider:SetValue(profile.buff.borderSize)

	controls.healthTextCheck:SetValue(profile.healthText.enabled)
	controls.healthTextPercentageCheck:SetValue(profile.healthText.percentage)
	controls.healthTextMaxCheck:SetValue(profile.healthText.displayMax)
	controls.executeThresholdSlider:SetValue(profile.healthText.executeThreshold or 0.2)
	controls.disableExecuteCheck:SetValue(not profile.healthText.executeThreshold)
	ColorPickerSetHexColor(controls.executeColor, profile.healthText.colorExecute)
	HealthText_OnValueChanged()

	controls.flashCheck:SetValue(profile.target.flash)
	controls.shineCheck:SetValue(profile.target.shine)
	controls.shineAlphaSlider:SetValue(profile.target.shineAlpha)
	controls.animationCheck:SetValue(profile.target.animation)
	controls.animationAlphaSlider:SetValue(profile.target.animationAlpha)
	controls.animationStyleDropdown:SetValue(profile.target.animationStyle)
	controls.animationOverlayCheck:SetValue(profile.target.animationOverlay)
	Shine_OnValueChanged()
	Animation_OnValueChanged()

	-- controls.performanceModeCheck:SetValue(profile.performanceMode)
	controls.targetAlwaysOnTop:SetValue(profile.targetAlwaysOnTop)
	controls.redHostileCheck:SetValue(profile.redHostile)
	controls.enemyPowerBarCheck:SetValue(profile.powerBarEnemy)
	controls.disableFactionIconCheck:SetValue(profile.disableFactionIcon)
	controls.friendlyClassColorCheck:SetValue(profile.friendlyClassColor)
	controls.friendlyClickThroughCheck:SetValue(profile.friendlyClickThrough)
	controls.disableFreindlyBarsCheck:SetValue(profile.disableFriendlyBars)
	controls.disableQuestObjectiveIconCheck:SetValue(profile.disableObjectiveIcon)
	controls.performanceModeDropdown:SetValue(profile.performanceMode)

	controls.range40Check:SetValue(GetCVar("nameplateMaxDistance") == "40")
	controls.clampTopCheck:SetValue(GetCVar("nameplateOtherTopInset") ~= "-1")
	controls.clampBottomCheck:SetValue(GetCVar("nameplateOtherBottomInset") ~= "-1")
	controls.showFriendlyNpcCheck:SetValue(GetCVar("nameplateShowFriendlyNPCs") ~= "0")

	controls.disableDefaultFiltersCheck:SetValue(profile.disableDefaultFilters)

end

function mainPanel:okay()
	profile.frameSize.W = controls.healthWidthSlider:GetValue()
	--profile.frameSize.H = controls.healthHeightSlider:GetValue()
	--profile.frameScale.x = controls.frameScaleHorizontal:GetValue()
	--profile.frameScale.y = controls.frameScaleVertical:GetValue()
	profile.healthBarHeight = controls.healthHeightSlider:GetValue()
	profile.castBarHeight = controls.castHeightSlider:GetValue()

	profile.frameMargin.y = controls.frameMarginY:GetValue()
	profile.raidIcon.size = controls.raidIconSlider:GetValue()
	profile.spellIcon.size = controls.spellIconSlider:GetValue()
	profile.barBackdrop.edgeSize = controls.borderWidthSlider:GetValue()

	local borderStyle = borderTypes[controls.borderStyleDropdown:GetValue() or 0]
	if borderStyle then
		profile.barBackdrop.edgeFile = borderStyle.file
		profile.borderColor[4] = borderStyle.alpha
		profile.castBarOffset = borderStyle.GetCastBarOffset(profile.barBackdrop.edgeSize)
	end

	if controls.unitFontDropdown then
		local fontName = controls.unitFontDropdown:GetValue()
		if fontName then
			profile.nameFont.name = (fontName ~= _G.DEFAULT) and controls.unitFontDropdown.list[fontName] or nil
		end
		profile.nameFont.size = controls.unitNameSlider:GetValue()
		profile.nameFont.outline = controls.unitNameOutlineCheck:GetValue() and "OUTLINE" or "NONE"
	end

	if controls.unitFontPlayerDropdown then
		local fontName = controls.unitFontPlayerDropdown:GetValue()
		if fontName then
			profile.nameFontPlayer.name = (fontName ~= _G.DEFAULT) and controls.unitFontPlayerDropdown.list[fontName] or nil
		end
		profile.nameFontPlayer.size = controls.unitNamePlayerSlider:GetValue()
		profile.nameFontPlayer.outline = controls.unitNamePlayerOutlineCheck:GetValue() and "OUTLINE" or "NONE"
	end

	if controls.spellFontDropdown then
		local fontName = controls.spellFontDropdown:GetValue()
		if fontName then
			profile.castFont.name = (fontName ~= _G.DEFAULT) and controls.spellFontDropdown.list[fontName] or nil
		end
		profile.castFont.size = controls.spellNameSlider:GetValue()
		profile.castFont.outline = controls.spellNameOutlineCheck:GetValue() and "OUTLINE" or "NONE"
	end

	local texture = controls.healthTextureDropdown and controls.healthTextureDropdown:GetValue()
	if texture then
		profile.barTexture = (fontName ~= BARTEXTURE_DEFAULT) and controls.healthTextureDropdown.list[texture] or nil
	end

	local texture = controls.castTextureDropdown and controls.castTextureDropdown:GetValue()
	if texture then
		profile.castBarTexture = (fontName ~= BARTEXTURE_DEFAULT) and controls.castTextureDropdown.list[texture] or nil
	end

	profile.buff.enabledEnemy = controls.showDebuffCheck:GetValue()
	profile.buff.enabledFriendly = controls.showBuffCheck:GetValue()
	profile.buff.countdownNumber = controls.showCountdownCheck:GetValue() and GetCVarBool("countdownForCooldowns")
	profile.disableOmnicc = controls.disableOmniccCheck:GetValue()
	profile.buff.size.H = controls.auraHeightSlider:GetValue()
	if profile.buff.size.H == addon.defaults.buff.size.H then
		profile.buff.size.W = addon.defaults.buff.size.W
	else
		profile.buff.size.W = floor(profile.buff.size.H * addon.defaults.buff.size.W / addon.defaults.buff.size.H)
	end
	profile.buff.borderSize = controls.auraBorderSlider:GetValue()

	profile.healthText.enabled = controls.healthTextCheck:GetValue()
	profile.healthText.percentage = controls.healthTextPercentageCheck:GetValue()
	profile.healthText.displayMax = controls.healthTextMaxCheck:GetValue()
	if controls.disableExecuteCheck:GetValue() then
		profile.healthText.executeThreshold = false
	else
		profile.healthText.executeThreshold = controls.executeThresholdSlider:GetValue()
	end
	profile.healthText.colorExecute = ColorPickerGetHexColor(controls.executeColor)

	profile.target.flash = controls.flashCheck:GetValue()
	profile.target.shine = controls.shineCheck:GetValue()
	profile.target.shineAlpha = controls.shineAlphaSlider:GetValue()
	profile.target.animation = controls.animationCheck:GetValue()
	profile.target.animationAlpha = controls.animationAlphaSlider:GetValue()
	profile.target.animationStyle = controls.animationStyleDropdown:GetValue()
	profile.target.animationOverlay = controls.animationOverlayCheck:GetValue()

	local value
	value = controls.range40Check:GetValue()
	if value == (GetCVar("nameplateMaxDistance") ~= "40") then
		addon:QueueSetCVar("nameplateMaxDistance", value and 40 or GetCVarDefault("nameplateMaxDistance"))
	end

	value = controls.clampTopCheck:GetValue()
	if value == (GetCVar("nameplateOtherTopInset") == "-1") then
		addon:QueueSetCVar("nameplateOtherTopInset", value and GetCVarDefault("nameplateOtherTopInset") or -1)
	end

	value = controls.clampBottomCheck:GetValue()
	if value == (GetCVar("nameplateOtherBottomInset") == "-1") then
		addon:QueueSetCVar("nameplateOtherBottomInset", value and GetCVarDefault("nameplateOtherBottomInset") or -1)
	end

	value = controls.showFriendlyNpcCheck:GetValue()
	if value ~= GetCVarBool("nameplateShowFriendlyNPCs") then
		addon:QueueSetCVar("nameplateShowFriendlyNPCs", value and 1 or 0)
	end

	profile.redHostile = controls.redHostileCheck:GetValue()
	profile.friendlyClassColor = controls.friendlyClassColorCheck:GetValue()
	profile.powerBarEnemy = controls.enemyPowerBarCheck:GetValue()
	profile.friendlyClickThrough = controls.friendlyClickThroughCheck:GetValue()
	profile.disableFriendlyBars = controls.disableFreindlyBarsCheck:GetValue()
	profile.targetAlwaysOnTop = controls.targetAlwaysOnTop:GetValue()
	profile.disableFactionIcon = controls.disableFactionIconCheck:GetValue()
	profile.disableObjectiveIcon = controls.disableQuestObjectiveIconCheck:GetValue()
	profile.performanceMode = controls.performanceModeDropdown:GetValue()

	local value = controls.disableDefaultFiltersCheck:GetValue() and true
	if profile.disableDefaultFilters ~= value then
		profile.disableDefaultFilters = value
		addon:ClearFilterCacheAll()
	end

	options:SaveUserFilters()
	options:ClearUserFilters()
	addon:LoadProfile(true)
end

function mainPanel:default()
	db:ResetProfile()
	mainPanel:refresh()
	mainPanel:okay()
end

function filterPanel:refresh()
	options:RefreshFilters()
end

function filterPanel:okay()

end

function filterPanel:cancel()
	options:ClearUserFilters()
end

function filterPanel:default()
	HideUIPanel(ColorPickerFrame)
	options:ResetUserFilters()
	options:ClearUserFilters()
	addon:UpdateFilter()
	addon:RestyleNamePlates()
	filterPanel:refresh()
end

local function NOP() end

local function Slider_OnMouseWheel(frame, delta)
	local widget = frame.obj
	if not widget.disabled then
		widget:SetValue(math.min(math.max(widget.value + (widget.step * delta), widget.min), widget.max))
	end
end

local function SliderEditbox_OnFocusGained(frame)
	frame.obj.frame:EnableMouseWheel(true)
	frame.obj.frame:SetScript("OnMouseWheel", Slider_OnMouseWheel)
end

local function SliderEditbox_OnFocusLost(frame)
	frame.obj.frame:EnableMouseWheel(false)
	frame.obj.frame:SetScript("OnMouseWheel", nil)
end

local function CreateGroup(name, title, parent)
	local group = GUI:Create("InlineGroup")
	group.frame:SetParent(parent or mainPanel)
	group:SetTitle(title)
	group:SetLayout("Flow")
	group.frame:EnableMouseWheel(true)
	group.frame:SetScript("OnMouseWheel", NOP)
	--group.frame:Show()
	controls[name] = group
	return group
end

local function CreateGroup2(name, title)
	local group = GUI:Create("InlineGroup")
	group:SetFullWidth(true)
	group:SetTitle(title)
	group:SetLayout("Flow")
	-- group.frame:EnableMouseWheel(true)
	-- group.frame:SetScript("OnMouseWheel", NOP)
	controls[name] = group
	return group
end

local function CreateSlider(name, label, min, max, step, width, onValueChanged)
	if not onValueChanged and type(width) == "function" then
		onValueChanged = width
		width = nil
	end
	local slider = GUI:Create("Slider")
	if not width or width < 1 then
		slider:SetRelativeWidth(width or 0.242)
	else
		slider:SetWidth(width)
	end
	slider:SetLabel(label)
	slider:SetSliderValues(min, max, step)
	-- slider.frame:EnableMouseWheel(true)
	-- slider.frame:SetScript("OnMouseWheel", Slider_OnMouseWheel)
	slider.editbox:SetScript("OnEditFocusGained", SliderEditbox_OnFocusGained)
	slider.editbox:SetScript("OnEditFocusLost", SliderEditbox_OnFocusLost)
	if type(onValueChanged) == "function" then
		slider:SetCallback("OnValueChanged", onValueChanged)
	end
	controls[name] = slider
	return slider
end

local function CreateHeader(title, name)
	local header = GUI:Create("Heading")
	header:SetFullWidth(true)
	if title then
		header:SetText(title)
	end
	if name then
		controls[name] = header
	end
	return header
end

local function CreateCheckBox(name, label, description, width, onValueChanged)
	local check = GUI:Create("CheckBox")
	if width > 0 then
		check:SetRelativeWidth(width)
	else
		check:SetWidth(-width)
	end
	check:SetLabel(label)
	if description then
		check:SetDescription(description)
	end
	if type(onValueChanged) == "function" then
		check:SetCallback("OnValueChanged", onValueChanged)
	end
	controls[name] = check
	return check
end

function options:CreateWidget()
	local titleText = mainPanel:CreateFontString(nil, nil, "GameFontNormalLarge")
	titleText:SetText(mainPanel.name)
	titleText:SetPoint("TOPLEFT", mainPanel, FRAME_MARGIN, -FRAME_MARGIN)

	local applyButton = GUI:Create("Button") -- Apply
	applyButton.frame:SetParent(mainPanel)
	applyButton:SetWidth(110)
	applyButton.frame:SetHeight(20)
	applyButton:SetPoint("TOPRIGHT", mainPanel, -7, -6)
	applyButton:SetText(_G.APPLY)
	applyButton:SetCallback("OnClick", function()
		mainPanel:okay()
		-- mainPanel:refresh()
	end)
	applyButton.frame:Show()

	local filtersButton = GUI:Create("Button") -- Filters
	filtersButton.frame:SetParent(mainPanel)
	filtersButton:SetWidth(110)
	filtersButton.frame:SetHeight(applyButton.frame:GetHeight())
	filtersButton:SetPoint("RIGHT", applyButton.frame, "LEFT", -5, 0)
	filtersButton:SetText(filterPanel.name)
	filtersButton:SetCallback("OnClick", function()
		InterfaceOptionsFrame_OpenToCategory(filterPanel)
	end)
	filtersButton.frame:Show()

	local mainGroup = GUI:Create("ScrollFrame")
	-- group:AddChild(filterGroup)
	mainGroup.frame:SetParent(mainPanel)
	mainGroup:SetLayout("Flow")
	-- mainGroup:SetFullHeight(true)
	-- mainGroup:SetFullWidth(true)
	mainGroup:SetPoint("TOPLEFT", mainPanel, FRAME_MARGIN, -36)
	mainGroup:SetPoint("TOPRIGHT", mainPanel, -6, -36)
	mainGroup:SetPoint("BOTTOM", mainPanel, 0, 7)
	--mainGroup.frame:SetHeight(485)
	mainGroup.frame:Show()

	local group = CreateGroup2("styleGroup", "Style")
	mainGroup:AddChild(group)
	-- group:SetPoint("TOPLEFT", mainPanel, FRAME_MARGIN, -136)
	-- group:SetPoint("TOPRIGHT", mainPanel, -FRAME_MARGIN, -136)
	-- group.frame:Show()

	group:AddChild(CreateSlider("healthWidthSlider", "Frame Width", 50, 200, 0.5))
	--group:AddChild(CreateSlider("healthHeightSlider", "Height (Health)", 3, 30, 0.1))
	--group:AddChild(CreateSlider("frameScaleHorizontal", "Frame Width", 0.5, 2.0, 0.01))
	--group:AddChild(CreateSlider("frameScaleVertical", "Scale Y", 0.5, 5.0, 0.05))
	group:AddChild(CreateSlider("frameMarginY", "Frame Margin", 0, 60, 1))
	group:AddChild(CreateSlider("healthHeightSlider", "Health Bar Height", 3, 25, 0.1))
	group:AddChild(CreateSlider("castHeightSlider", "Cast Bar Height", 3, 25, 0.1))
	--group:AddChild(CreateHeader())
	group:AddChild(CreateSlider("raidIconSlider", "Raid Icon Size", 5, 40, 0.5))
	--group:AddChild(CreateSlider("numberSlider", "Number Size", 5, 30, 0.1))
	group:AddChild(CreateSlider("spellIconSlider", "Spell Icon Size", 5, 30, 0.5))
	group:AddChild(CreateSlider("borderWidthSlider", "Border Width", 1, 15, 0.5))

	local borderStyleDropdown = GUI:Create("Dropdown")
	borderStyleDropdown:SetLabel("Border Style")
	borderStyleDropdown:SetRelativeWidth(0.235)
	local list = {}
	for i = 1, #borderTypes do list[i] = borderTypes[i].name end
	borderStyleDropdown:SetList(list)
	controls.borderStyleDropdown = borderStyleDropdown
	group:AddChild(borderStyleDropdown)

	if AceGUIWidgetLSMlists and AceGUIWidgetLSMlists.font then
		group:AddChild(CreateHeader("Fonts"))

		local unitFontDropdown = GUI:Create("LSM30_Font")
		unitFontDropdown:SetLabel("Unit Name")
		unitFontDropdown:SetRelativeWidth(0.55)
		local unitFontList = {[_G.DEFAULT] = _G.GameFontWhite:GetFont()}
		for k, v in pairs(AceGUIWidgetLSMlists.font) do
			unitFontList[k] = v
		end
		unitFontDropdown:SetList(unitFontList)
		unitFontDropdown:SetCallback("OnValueChanged", function(widget, event, key)
			widget:SetValue(key)
		end)
		controls.unitFontDropdown = unitFontDropdown
		group:AddChild(unitFontDropdown)
		group:AddChild(CreateSlider("unitNameSlider", "Size", 5, 30, 0.1, 0.24))
		group:AddChild(CreateCheckBox("unitNameOutlineCheck", "Outline", nil, 0.18))

		local spellFontDropdown = GUI:Create("LSM30_Font")
		spellFontDropdown:SetLabel("Spell Name and Status Text")
		spellFontDropdown:SetRelativeWidth(0.55)
		local spellFontList = {[_G.DEFAULT] = _G.GameFontWhiteTiny:GetFont()}
		for k, v in pairs(AceGUIWidgetLSMlists.font) do
			spellFontList[k] = v
		end
		spellFontDropdown:SetList(spellFontList)
		spellFontDropdown:SetCallback("OnValueChanged", function(widget, event, key)
			widget:SetValue(key)
		end)
		controls.spellFontDropdown = spellFontDropdown
		group:AddChild(spellFontDropdown)
		group:AddChild(CreateSlider("spellNameSlider", "Size", 5, 30, 0.1, 0.24))
		group:AddChild(CreateCheckBox("spellNameOutlineCheck", "Outline", nil, 0.18))

		local unitFontPlayerDropdown = GUI:Create("LSM30_Font")
		unitFontPlayerDropdown:SetLabel("Friendly Player"..ORANGE_FONT_COLOR_CODE.." *only used when no health bar"..FONT_COLOR_CODE_CLOSE)
		unitFontPlayerDropdown:SetRelativeWidth(0.55)
		local unitFontList = {[_G.DEFAULT] = _G.GameFontWhite:GetFont()}
		for k, v in pairs(AceGUIWidgetLSMlists.font) do
			unitFontList[k] = v
		end
		unitFontPlayerDropdown:SetList(unitFontList)
		unitFontPlayerDropdown:SetCallback("OnValueChanged", function(widget, event, key)
			widget:SetValue(key)
		end)
		controls.unitFontPlayerDropdown = unitFontPlayerDropdown
		group:AddChild(unitFontPlayerDropdown)
		group:AddChild(CreateSlider("unitNamePlayerSlider", "Size", 5, 30, 0.1, 0.24))
		group:AddChild(CreateCheckBox("unitNamePlayerOutlineCheck", "Outline", nil, 0.18))
	end

	if AceGUIWidgetLSMlists and AceGUIWidgetLSMlists.statusbar then
		group:AddChild(CreateHeader("Textures"))
		local healthTextureDropdown = GUI:Create("LSM30_Statusbar")
		healthTextureDropdown:SetLabel("Health Bar")
		healthTextureDropdown:SetRelativeWidth(0.45)
		local healthTexturelist = {[BARTEXTURE_DEFAULT] = addon.defaults.castBarTexture or addon.defaults.barTexture}
		for k, v in pairs(AceGUIWidgetLSMlists.statusbar) do
			healthTexturelist[k] = v
		end
		healthTextureDropdown:SetList(healthTexturelist)
		healthTextureDropdown:SetCallback("OnValueChanged", function(widget, event, key)
			widget:SetValue(key)
		end)
		controls.healthTextureDropdown = healthTextureDropdown
		group:AddChild(healthTextureDropdown)

		local castTextureDropdown = GUI:Create("LSM30_Statusbar")
		castTextureDropdown:SetLabel("Cast Bar")
		castTextureDropdown:SetRelativeWidth(0.45)
		local castTexturelist = {[BARTEXTURE_DEFAULT] = addon.defaults.castBarTexture or addon.defaults.barTexture}
		for k, v in pairs(AceGUIWidgetLSMlists.statusbar) do
			castTexturelist[k] = v
		end
		castTextureDropdown:SetList(castTexturelist)
		castTextureDropdown:SetCallback("OnValueChanged", function(widget, event, key)
			widget:SetValue(key)
		end)
		controls.castTextureDropdown = castTextureDropdown
		group:AddChild(castTextureDropdown)
	end

	local group = CreateGroup2("aurasGroup", _G.AURAS)
	mainGroup:AddChild(group)
	group:AddChild(CreateCheckBox("showDebuffCheck", "Enemy Auras", nil, 0.45))
	group:AddChild(CreateCheckBox("showBuffCheck", "Friendly Auras", nil, 0.45))
	group:AddChild(CreateCheckBox("showCountdownCheck", "Countdown Numbers", nil, 0.45))
	group:AddChild(CreateCheckBox("disableOmniccCheck", "Disable OmniCC", nil, 0.45))
	group:AddChild(CreateSlider("auraHeightSlider", "Icon Height", 8, 20, 1))
	group:AddChild(CreateSlider("auraBorderSlider", "Border Width", 1, 3, 0.5))

	local group = CreateGroup2("healthTextGroup", "Health Text")
	group:AddChild(CreateCheckBox("healthTextCheck", "Display Health Text", nil, 0.9, HealthText_OnValueChanged))
	group:AddChild(CreateCheckBox("healthTextPercentageCheck", "Percentage", nil, 0.9))
	group:AddChild(CreateCheckBox("healthTextMaxCheck", "Maximum Health", nil, 0.9))
	group:AddChild(CreateCheckBox("disableExecuteCheck", "Disable Execute State", nil, 0.3, ExecuteColorCheck_OnValueChanged))
	group:AddChild(CreateSlider("executeThresholdSlider", "Execute Threshold", 0.1, 0.9, 0.01, 0.3))
	controls.executeThresholdSlider:SetIsPercent(true)
	local executeColor = GUI:Create("ColorPicker")
	executeColor:SetHasAlpha(false)
	executeColor:SetRelativeWidth(0.35)
	executeColor:SetLabel("Execute Color")
	controls.executeColor = executeColor
	group:AddChild(executeColor)
	mainGroup:AddChild(group)

	local group = CreateGroup2("targetGroup", "Target Emphasize")
	mainGroup:AddChild(group)
	-- group:SetPoint("TOPLEFT", controls.styleGroup.frame, "BOTTOMLEFT", 0, -FRAME_SPACE)
	-- group:SetPoint("TOPRIGHT", controls.styleGroup.frame, "BOTTOMRIGHT", 0, -FRAME_SPACE)
	group:AddChild(CreateCheckBox("flashCheck", "Flash", nil, 0.15))
	group:AddChild(CreateCheckBox("shineCheck", "Shine", nil, 0.15, Shine_OnValueChanged))
	group:AddChild(CreateSlider("shineAlphaSlider", "Shine Alpha", 0.1, 1.0, 0.05))
	group:AddChild(CreateHeader())
	group:AddChild(CreateCheckBox("animationCheck", "Animation", nil, 0.2, Animation_OnValueChanged))
	group:AddChild(CreateSlider("animationAlphaSlider", "Animation Alpha", 0.01, 1.0, 0.01, 155))

	local animationStyleDropdown = GUI:Create("Dropdown")
	animationStyleDropdown:SetLabel("Animation Style")
	animationStyleDropdown:SetRelativeWidth(0.2)
	animationStyleDropdown:SetList({default = _G.DEFAULT, fit = "Fit", wide = "Wide", full = "Full"}, {"default", "fit", "wide", "full"})
	controls.animationStyleDropdown = animationStyleDropdown
	group:AddChild(animationStyleDropdown)

	group:AddChild(CreateCheckBox("animationOverlayCheck", "Overlay", nil, 0.15))
	-- group.frame:Show()

	local group = CreateGroup2("commonGroup", "Common")
	mainGroup:AddChild(group)
	-- group:SetPoint("TOPLEFT", controls.targetGroup.frame, "BOTTOMLEFT", 0, -FRAME_SPACE)
	-- group:SetPoint("TOPRIGHT", controls.targetGroup.frame, "BOTTOMRIGHT", 0, -FRAME_SPACE)

	group:AddChild(CreateCheckBox("redHostileCheck", "Use red color for hostile NPC", nil, 0.9))
	group:AddChild(CreateCheckBox("friendlyClassColorCheck", "Use class colors for friendly players", nil, 0.9))
	group:AddChild(CreateCheckBox("enemyPowerBarCheck", "Display power bar for hostile players", nil, 0.9))
	group:AddChild(CreateCheckBox("friendlyClickThroughCheck", "Disable Mouse Events on the friendly units", nil, 0.9))
	group:AddChild(CreateCheckBox("disableFreindlyBarsCheck", "Do not display a health bar for frinedly units", nil, 0.9))
	group:AddChild(CreateCheckBox("disableFactionIconCheck", "Do not display a faction icon in PvP Battle Zone", nil, 0.9))
	group:AddChild(CreateCheckBox("disableQuestObjectiveIconCheck", "Disable Quest Objective Indicator", nil, 0.9))
	group:AddChild(CreateCheckBox("targetAlwaysOnTop", "Target is always on top", nil, 0.9))

	local performanceModeDropdown = GUI:Create("Dropdown")
	performanceModeDropdown:SetLabel("Performance Mode")
	performanceModeDropdown:SetRelativeWidth(0.2)
	performanceModeDropdown:SetList({[-1] = "Auto", [0] = _G.VIDEO_OPTIONS_DISABLED, [1] = _G.VIDEO_OPTIONS_ENABLED}, {-1, 0, 1})
	controls.performanceModeDropdown = performanceModeDropdown
	group:AddChild(performanceModeDropdown)

	local label = GUI:Create("Label")
	label:SetRelativeWidth(0.013)
	group:AddChild(label)

	local label = GUI:Create("Label")
	label:SetText("Fully suspend the hidden Blizzard Nameplates, also disable Personal Resource. If enable this improve performance. "..NORMAL_FONT_COLOR_CODE.."Currently "..(addon.performanceMode and _G.VIDEO_OPTIONS_ENABLED or _G.VIDEO_OPTIONS_DISABLED)..FONT_COLOR_CODE_CLOSE..".\n"..RELOAD)
	label:SetRelativeWidth(0.77)
	group:AddChild(label)

	-- group.frame:Show()

	local group = CreateGroup2("advancedGroup", "Advanced Control (Character Specific)")
	mainGroup:AddChild(group)
	group:AddChild(CreateCheckBox("range40Check", "Show within 40 yd", nil, 0.9))
	group:AddChild(CreateCheckBox("showFriendlyNpcCheck", "Show friendly NPC", nil, 0.9))
	group:AddChild(CreateCheckBox("clampTopCheck", "Clamp to the screen top", nil, 0.9))
	group:AddChild(CreateCheckBox("clampBottomCheck", "Clamp to the screen bottom", nil, 0.9))
end

local NpcNames do
	local tooltip = CreateFrame("GameTooltip")
	tooltip.leftLines = {}
	tooltip.rightLines = {}
	for i = 1, 2 do
		local L,R = tooltip:CreateFontString(), tooltip:CreateFontString()
		L:SetFontObject(GameFontNormal)
		R:SetFontObject(GameFontNormal)
		tooltip:AddFontStrings(L,R)
		tooltip.leftLines[i] = L
	end
	local FORMAT_TOOLTIP_LEVEL = "^"..TOOLTIP_UNIT_LEVEL:gsub("%%s", ".+")
	local instanceID = select(8, GetInstanceInfo())
	local FORMAT_GUID_NPC = "unit:Creature-0-1-1-12345-%d-0000000000"
	NpcNames = setmetatable({}, {__index =
		function(self, key)
			tooltip:SetOwner(_G.UIParent, "ANCHOR_NONE")
			tooltip:SetHyperlink(FORMAT_GUID_NPC:format(key))
			local value = tooltip.leftLines[1]:GetText()
			tooltip:Hide()
			if value then
				self[key] = value
			end
			return value or _G.UNKNOWN
		end
	})
end

function options:ResetUserFilters()
	if _G.NamePlateKAIFilter then
		for map, filter in pairs(_G.NamePlateKAIFilter) do
			if filter.recolor then
				for name, color in pairs(filter.recolor) do
					if type(color) == "table" then
						wipe(color)
					end
				end
				wipe(filter.recolor)
			end
			wipe(filter)
			--addon:ClearFilterCache(map)
		end
		wipe(_G.NamePlateKAIFilter)
	end
	addon:ClearFilterCacheAll()
end

function options:SaveUserFilters()
	if not _G.NamePlateKAIFilter then
		_G.NamePlateKAIFilter = {}
	end
	local db = _G.NamePlateKAIFilter

	for map, filter in pairs(userFilters) do
		if filter.modified then
			if not db[map] then db[map] = { recolor = {} } end
			if not db[map].recolor then db[map].recolor = {} end
			wipe(db[map].recolor)
			for name, color in pairs(filter.color) do
				db[map].recolor[name] = color
			end
			if next(db[map].recolor) == nil then
				wipe(db[map])
				db[map] = nil
			end
			addon:ClearFilterCache(map)
			filter.modified = nil
		end
	end
	if not next(db) then
		_G.NamePlateKAIFilter = nil
	end
end

function options:ClearUserFilters()
	for map, filter in pairs(userFilters) do
		if filter.color then
			wipe(filter.color)
		end
		wipe(filter)
	end
	wipe(userFilters)

	controls.filterGroup:ReleaseChildren()
	wipe(filterControls)
end

local function GetColoredText(text, color)
	return ("|cff%.2x%.2x%.2x%s|r"):format(color[1] * 255, color[2] * 255, color[3] * 255, text)
end

local function GetColoredSqure(text, color)
	return ("|T130871:0:0:0:0:8:8:0:8:0:8:%d:%d:%d|t"):format(color[1] * 255, color[2] * 255, color[3] * 255)..(text and GetColoredText(text, color) or "")
end

local function LoadZoneFilter(map)
	if not addon.defaultFilters[map] then return end
	if not zoneFilters[map] then
		local t = {}
		for _, difficulty in pairs({"*", "Heroic", "Challenge", "Mythic"}) do
			if difficulty == "*" or addon.defaultFilters[map][difficulty] then
				local filter = addon:LoadFilter(addon.defaultFilters[map], difficulty)
				for k, v in pairs(filter.color) do
					if not t[k] then t[k] = {} end
					t[k][difficulty] = v
				end
			end
		end
		zoneFilters[map] = t
	end
	return zoneFilters[map]
end

local function RefreshFilterNpcNames()
	for k, v in pairs(filterControls) do
		if k:find("\\nameLabel$") then
			local id = v:GetUserData("id")
			if type(id) == "number" then
				v:SetText(NpcNames[id])
			end
		end
	end
end

local filterOrder = {}
function options:RefreshFilters()
	controls.filterGroup:ReleaseChildren()
	wipe(filterControls)
	local map = controls.filterZoneDropdown:GetValue()
	if not map then
		SetMapToCurrentZone()
		map = GetCurrentMapAreaID() or "default"
		if not controls.filterZoneDropdown.list[map] then
			map = "default"
		end
		controls.filterZoneDropdown:SetValue(map)
	end

	controls.filterGroup:PauseLayout()

	local zoneFilter = LoadZoneFilter(map)

	wipe(filterOrder)
	local order = addon.defaultFilters[map] and addon.defaultFilters[map].order
	if type(order) == "table" then
		local id
		for i, v in ipairs(order) do
			id = type(v) == "string" and tonumber(v)
			if id then
				tinsert(filterOrder, id < 0 and EJ_GetEncounterInfo(-id) or EJ_GetSectionInfo(id) or id)
			else
				tinsert(filterOrder, v)
			end
		end
	end

	local header = GUI:Create("Label")
	header:SetFullWidth(true)
	controls.filterGroup:AddChild(header)

	if not userFilters[map] then
		userFilters[map] = { color = {} }
		addon:LoadUserFilter(map, userFilters[map], true)
	end

	local count = 0

	for name, v in pairs(zoneFilter) do
		if not tContains(filterOrder, name) then
			options:AddFilterWidget(map, name, zoneFilter)
			count = count + 1
		end
	end

	local header
	for _, name in ipairs(filterOrder) do
		headerText = type(name) == "string" and name:match("h\\(.+)")
		if headerText then
			if tonumber(headerText) then
				headerText = EJ_GetEncounterInfo(headerText) or headerText
			end
			local header = GUI:Create("Heading")
			header:SetFullWidth(true)
			if headerText ~= "-" then
				header:SetText(headerText)
			end
			controls.filterGroup:AddChild(header)
		elseif zoneFilter[name] then
			options:AddFilterWidget(map, name, zoneFilter)
			count = count + 1
		end
	end

	--for name, v in pairs(zoneFilter) do
	--	options:AddFilterWidget(map, name, zoneFilter)
	--	count = count + 1
	--end

	if count > 0 then
		local header = GUI:Create("Heading")
		header:SetFullWidth(true)
		controls.filterGroup:AddChild(header)
	end

	for name, v in pairs(userFilters[map].color) do
		if not zoneFilter[name] then
			options:AddFilterWidget(map, name)
			count = count + 1
		end
	end

	if count > 0 then
		C_Timer.After(1, RefreshFilterNpcNames)
	else
		local noneLabel = GUI:Create("Label")
		noneLabel:SetRelativeWidth(0.3)
		noneLabel:SetFontObject(GameFontNormal)
		noneLabel:SetText(_G.NONE)
		controls.filterGroup:AddChild(noneLabel)
		local header = GUI:Create("Heading")
		header:SetFullWidth(true)
		controls.filterGroup:AddChild(header)
	end

	options:AddFilterButton()

	controls.filterGroup:ResumeLayout()
	controls.filterGroup:DoLayout()
end

local function GetZoneList()
	local list = {}
	local order = {}
	local name
	for map, filter in pairs(addon.defaultFilters) do
		if type(filter.zone) == "number" then
			name = EJ_GetInstanceInfo(filter.zone)
		else
			name = filter.zone
		end
		if not name then
			name = _G.UNKNOWN.."["..map.."]"
		end
		if type(filter.category) == "number" then
			if filter.category > 700 then
				name = "|cff44ff44"..name.."|r"
			elseif filter.category > 600 then
				name = "|cffffcc44"..name.."|r"
			elseif filter.category > 500 then
				name = "|cff44ffcc"..name.."|r"
			elseif filter.category > 400 then
				name = "|cffff4444"..name.."|r"
			end
			if (filter.category % 100) > 90 then
				name = name.." (".._G.PVP..")"
			elseif (filter.category % 100) > 50 then
				name = name.." (".._G.RAID..")"
			end
		end
		tinsert(order, map)
		list[map] = name
	end
	local p1, p2
	table.sort(order, function(v1, v2)
		v1 = tonumber(v1) or v1
		v2 = tonumber(v2) or v2
		p1 = tonumber(addon.defaultFilters[v1] and addon.defaultFilters[v1].category or 10000)
		p2 = tonumber(addon.defaultFilters[v2] and addon.defaultFilters[v2].category or 10000)
		if math.floor(p1 / 100) == math.floor(p2 / 100) then
			return p1 < p2
		else
			return math.floor(p1 / 100) > math.floor(p2 / 100)
		end
	end)
	return list, order
end

local function GetFilterDefaultText(zoneFilter, name)
	local text
	local color = zoneFilter[name]["*"]
	if color then
		text = GetColoredSqure(_G.DEFAULT, color)
	else
		text = GetColoredText(_G.DEFAULT, profile.barColors.tapped)
	end
	text = text.."  "
	color = zoneFilter[name]["Heroic"]
	if color then
		text = text..GetColoredSqure("H ", color)
	end
	color = zoneFilter[name]["Challenge"]
	if color then
		text = text..GetColoredSqure("C ", color)
	end
	color = zoneFilter[name]["Mythic"]
	if color then
		text = text..GetColoredSqure("M ", color)
	end
	return text
end

local function FilterColorDropdown_Refresh(dropdown)
	local map = dropdown:GetUserData("map")
	local name = dropdown:GetUserData("id")
	local value = "default"
	local colorPicker = filterControls[name.."\\colorPicker"]
	--if not colorPicker then return end
	if userFilters[map] then
		local color = userFilters[map].color[name]
		if color == false then
			value = "disable"
		elseif type(color) == "number" then
			value = color
		elseif type(color) == "table" then
			value = "custom"
			colorPicker = colorPicker or options:AddFilterColorPicker(map, name)
			colorPicker:SetColor(color[1], color[2], color[3])
		end
	end
	dropdown:SetValue(value)
	if colorPicker then
		colorPicker.frame:SetShown(value == "custom")
	end
end

local filterColors = {
	[1] = GetColoredSqure("1 Light Yellow", profile.filterColors[1]),
	[2] = GetColoredSqure("2 Light Purple", profile.filterColors[2]),
	[3] = GetColoredSqure("3 Light Blue", profile.filterColors[3]),
	[4] = GetColoredSqure("4 (no aggro)", profile.filterColors[4]),
	[5] = GetColoredSqure("5 (no aggro)", profile.filterColors[5]),
	[6] = GetColoredSqure("6 (no aggro)", profile.filterColors[6]),
	disable = _G.DISABLE,
	custom = _G.CUSTOM,
}
local colorOrder = {"default", "disable", 1, 2, 3, 4, 5, 6, "custom"}
local userOrder = {"disable", 1, 2, 3, 4, 5, 6, "custom", "remove"}

--do
--	for name, c in pairs(RAID_CLASS_COLORS) do
--		filterColors[name] = GetColoredSqure(name, {c.r, c.g, c.b})
--		tinsert(colorOrder, name)
--	end
--end

local function FilterColorDropdown_OnValueChanged(dropdown, event, key)
	HideUIPanel(ColorPickerFrame)
	local map = dropdown:GetUserData("map")
	local name = dropdown:GetUserData("id")
	local userFilter = userFilters[map]
	if not userFilter then return end

	if key == "default" then
		userFilter.color[name] = nil
	elseif key == "disable" then
		userFilter.color[name] = false
	elseif key == "custom" then
		if type(userFilter.color[name]) ~= "table" then
			local colorPicker = filterControls[name.."\\colorPicker"]
			if colorPicker then
				userFilter.color[name] = {colorPicker.r, colorPicker.g, colorPicker.b}
			else
				userFilter.color[name] = {unpack(profile.filterColors[1], 1, 3)}
			end
		end
	elseif key == "remove" then
		userFilter.color[name] = nil
		userFilter.modified = true
		if not zoneFilters[map][name] then
			options:RemoveFilterWidget(map, name)
			controls.filterGroup:DoLayout()
			return
		end
	elseif type(key) == "number" then
		userFilter.color[name] = key
	else
		return
	end
	userFilter.modified = true
	FilterColorDropdown_Refresh(dropdown)
end

local function FilterZoneDropdown_OnValueChanged(widget, event, key)
	HideUIPanel(ColorPickerFrame)
	options:RefreshFilters()
end

local function FilterColorPicker_OnValueConfirmed(colorPicker, event, r, g, b, a)
	local map = colorPicker:GetUserData("map")
	local name = colorPicker:GetUserData("id")
	local color = userFilters[map] and userFilters[map].color[name]
	if type(color) == "table" and (color[1] ~= r or color[2] ~= g or color[3] ~= b) then
		wipe(color)
		color[1] = r
		color[2] = g
		color[3] = b
		userFilters[map].modified = true
	end
end

local function FilterResetButton_OnClick(button, event)
	HideUIPanel(ColorPickerFrame)
	local map = controls.filterZoneDropdown:GetValue()
	if not map then return end
	if not userFilters[map] then
		userFilters[map] = { color = {} }
	end
	local userFilter = userFilters[map]
	wipe(userFilter.color)
	userFilter.modified = true
	options:RefreshFilters()
end

function options:CreateFiltersWidget()
	local filterTitleText = filterPanel:CreateFontString(nil, nil, "GameFontNormalLarge")
	filterTitleText:SetText(mainPanel.name.." - "..filterPanel.name)
	filterTitleText:SetPoint("TOPLEFT", filterPanel, FRAME_MARGIN, -FRAME_MARGIN)

	local checkBox = CreateCheckBox("disableDefaultFiltersCheck", "Disable all bult-in filters", nil, -210)
	checkBox.frame:SetParent(filterPanel)
	checkBox:SetPoint("TOPRIGHT", filterPanel, -FRAME_MARGIN, -FRAME_MARGIN)
	checkBox.frame:Show()

	local group = CreateGroup("filterGroup", "", filterPanel)
	group:SetPoint("TOPLEFT", filterPanel, FRAME_MARGIN, -26)
	group:SetPoint("BOTTOMRIGHT", filterPanel, -FRAME_MARGIN, FRAME_MARGIN -4)
	group.frame:Show()

	local filterZoneDropdown = GUI:Create("Dropdown")
	filterZoneDropdown.text:SetJustifyH("CENTER")
	--filterZoneDropdown:SetFullWidth(true)
	filterZoneDropdown:SetRelativeWidth(0.84)
	filterZoneDropdown:SetList(GetZoneList())
	filterZoneDropdown:SetCallback("OnValueChanged", FilterZoneDropdown_OnValueChanged)
	hooksecurefunc(filterZoneDropdown.pullout, "Open", function()
		HideUIPanel(ColorPickerFrame)
	end)
	controls.filterZoneDropdown = filterZoneDropdown
	group:AddChild(filterZoneDropdown)

	local resetButton = GUI:Create("Button") -- Reset
	resetButton:SetRelativeWidth(0.15)
	resetButton:SetText(_G.RESET)
	resetButton:SetCallback("OnClick", FilterResetButton_OnClick)
	controls.resetButton = resetButton
	group:AddChild(resetButton)

	local filterGroup = GUI:Create("ScrollFrame")
	group:AddChild(filterGroup)
	filterGroup:SetLayout("Flow")
	filterGroup:SetFullHeight(true)
	filterGroup:SetFullWidth(true)
	controls.filterGroup = filterGroup
end

function options:AddFilterWidget(map, name, zoneFilter)
	if not name then return end

	local nextWidget = filterControls.nameEdit

	local idLabel = GUI:Create("Label")
	idLabel:SetUserData("id", name)
	idLabel:SetRelativeWidth(0.13)
	idLabel:SetFontObject(GameFontNormal)
	idLabel:SetText(type(name) == "number" and name or nil)
	filterControls[name.."\\idLabel"] = idLabel
	controls.filterGroup:AddChild(idLabel, nextWidget)

	local nameLabel = GUI:Create("Label")
	nameLabel:SetUserData("id", name)
	nameLabel:SetRelativeWidth(0.52)
	nameLabel:SetFontObject(GameFontNormal)
	local text = name
	if type(name) == "number" then
		text = NpcNames[name]
	end
	nameLabel:SetText(text or _G.UNKNOWN)
	filterControls[name.."\\nameLabel"] = nameLabel
	controls.filterGroup:AddChild(nameLabel, nextWidget)

	local colorDropdown = GUI:Create("Dropdown")
	colorDropdown:SetUserData("id", name)
	colorDropdown:SetUserData("map", map)
	colorDropdown:SetRelativeWidth(0.27)
	local text
	local list, order
	if zoneFilter then
		list = {default = GetFilterDefaultText(zoneFilter, name)}
		order = colorOrder
	else
		list = {remove = _G.DELETE}
		order = userOrder
	end
	for k, v in pairs(filterColors) do
		list[k] = v
	end
	colorDropdown:SetList(list, order)
	colorDropdown:SetCallback("OnValueChanged", FilterColorDropdown_OnValueChanged)
	filterControls[name.."\\colorDropdown"] = colorDropdown
	controls.filterGroup:AddChild(colorDropdown, nextWidget)

	local footer = GUI:Create("Label")
	footer:SetUserData("id", name)
	footer:SetFullWidth(true)
	filterControls[name.."\\footer"] = footer
	controls.filterGroup:AddChild(footer, nextWidget)

	--controls.filterGroup:AddChildren(idLabel, nameLabel, colorDropdown, footer)

	FilterColorDropdown_Refresh(colorDropdown)
end

function options:AddFilterColorPicker(map, name)
	local colorPicker = GUI:Create("ColorPicker")
	colorPicker:SetUserData("id", name)
	colorPicker:SetUserData("map", map)
	colorPicker:SetRelativeWidth(0.04)
	colorPicker:SetColor(0, 0, 0)
	colorPicker:SetHasAlpha(false)
	colorPicker:SetCallback("OnValueConfirmed", FilterColorPicker_OnValueConfirmed)
	filterControls[name.."\\colorPicker"] = colorPicker
	controls.filterGroup:AddChild(colorPicker, filterControls[name.."\\footer"])
	return colorPicker
end

function options:AddFilter(map, name)
	if not map then return end
	if not name then return end
	name = tonumber(name) or name

	local zoneFilter = zoneFilters[map]
	if not zoneFilter then return end
	if zoneFilter[name] then
		return "Filter already exists: "..name
	end

	if not userFilters[map] then
		userFilters[map] = { color = {} }
	end
	local userFilter = userFilters[map]
	if userFilter.color[name] ~= nil then
		return "Filter already exists: "..name
	end
	userFilter.color[name] = false
	userFilter.modified = true
	options:AddFilterWidget(map, name)
end

local function FilterAddButton_OnClick(widget, event, ...)
	HideUIPanel(ColorPickerFrame)
	local name = filterControls.nameEdit:GetText()
	if name and name ~= "" then
		local errorMsg = options:AddFilter(controls.filterZoneDropdown:GetValue(), name)
		if errorMsg then
			filterControls.nameEdit:SetLabel(errorMsg)
		else
			filterControls.nameEdit:SetText()
			C_Timer.After(0.3, RefreshFilterNpcNames)
		end
	end
end

function options:AddFilterButton()
	local nameEdit = GUI:Create("EditBox")
	nameEdit:SetRelativeWidth(0.5)
	nameEdit:SetLabel("Enter NPC ID or Name")
	filterControls.nameEdit = nameEdit
	controls.filterGroup:AddChild(nameEdit)

	local addButton = GUI:Create("Button")
	addButton:SetRelativeWidth(0.22)
	addButton:SetText(ADD_FILTER)
	addButton:SetCallback("OnClick", FilterAddButton_OnClick)
	filterControls.addButton = addButton
	controls.filterGroup:AddChild(addButton)
end

function options:RemoveFilterWidget(map, name)
	for k, v in pairs(filterControls) do
		if k:find(name.."\\", 1, true) == 1 then
			filterControls[k] = nil
			v:Release()
			tDeleteItem(controls.filterGroup.children, v)
		end
	end
end

if not mainPanel:GetParent() then
	InterfaceOptions_AddCategory(mainPanel)
end
InterfaceOptions_AddCategory(filterPanel)
InterfaceAddOnsList_Update()
--InterfaceOptionsFrame_OpenToCategory(filterPanel)

mainPanel:refresh()
filterPanel:Hide()
--filterPanel:refresh()
C_Timer.After(0.5, filterPanel.refresh)
